<?php

namespace Drupal\spreadsheets;

use Drupal\file\Entity\File;
use Google\Service\Exception;

/**
 * Class for the service  javali_sheets.base .
 *
 * This service enables the creation,  * configuration and data
 * retrieval of a google sheet.
 */
class SpreadSheets {

  /**
   * The Google_Client object.
   *
   * @var \Google_Client
   */
  private $client;

  /**
   * The sheet id from config.
   *
   * @var string
   */
  private $sheetId;

  /**
   * The range from config.
   *
   * @var string
   */
  private $range;

  /**
   * The sheet name from config.
   *
   * @var string
   */
  private $sheetName;

  /**
   * JavaliSheets constructor.
   */
  public function __construct() {
    $config = \Drupal::config('spreadsheets.google_sheets_config');
    $this->client = $this->getClient();
    $this->sheetId = $config->get('sheet_id');
    $this->range = $config->get('range');
    $this->sheetName = $config->get('sheet_name');
  }

  /**
   * Function to get the Google client.
   *
   * @return \Google_Client
   *   the Google client
   *
   * @throws \Google\Service\Exception
   */
  public function getClient() {
    $config = \Drupal::config('spreadsheets.spreadsheetscredentials');
    $fid = $config->get('credentials_file')[0];
    $file = File::load($fid);
    $credentials_path = \Drupal::service('file_system')->realpath('private://' . $file->getFilename());
    $client = new \Google_Client();
    $client->setApplicationName('Google Sheets API PHP Quickstart');
    $client->setScopes([
      \Google_Service_Drive::DRIVE,
      \Google_Service_Sheets::SPREADSHEETS,
    ]);
    $client->setAuthConfig($credentials_path);
    $client->setAccessType('offline');
    $client->setApprovalPrompt('force');
    return $client;
  }

  /**
   * Get the API client and construct the service object.
   *
   * @return array
   *   an array with the sheet data
   */
  public function getdata($format = 'FORMATTED_VALUE') {
    $client = $this->client;
    $service = new \Google_Service_Sheets($client);
    $spreadsheetId = $this->sheetId;
    $name = $this->sheetName;
    $range = $this->range;
    $messenger = \Drupal::messenger();
    if (empty($this->sheetId)) {
      $messenger->addMessage('Sheet ID is required', 'error');
      return [];
    }
    if (empty($this->sheetName)) {
      $messenger->addMessage('Sheet Name is required', 'error');
      return [];
    }
    try {
      $sheetInfo = $service->spreadsheets->get($spreadsheetId);
      $allsheet_info = $sheetInfo['sheets'];
      $idCats = array_column($allsheet_info, 'properties');
      // Checks if the sheet exists.
      if (!$this->myArrayContainsWord($idCats, $name)) {
        $messenger->addMessage('Sheet not found', 'error');
        return [];
      }
      $options = [
        'valueRenderOption' => $format,
      ];
      $range_format = !empty($range) ? $this->sheetName . '!' . $range : $this->sheetName;
      $response = $service->spreadsheets_values->get($spreadsheetId, $range_format, $options);
      if (!empty($response->values)) {
        return $response->values;
      }
    }
    catch (Exception $e) {
      $message = json_decode($e->getMessage(), TRUE);
      $messenger->addMessage($message['error']['message'], 'error');
      return [];
    }
    $messenger->addMessage('An error ocurred!', 'error');
    return [];
  }

  /**
   * Set the sheet id.
   *
   * @param string $id
   *   The google sheet id.
   */
  public function setSheetId(string $id) {
    $this->sheetId = $id;
  }

  /**
   * Set the string range.
   *
   * @param string $range
   *   range (format ex A1:B2)
   */
  public function setRange(string $range) {
    $this->range = $range;
  }

  /**
   * Set the sheet name.
   *
   * @param string $name
   *   The name of the shee (ex:Sheet1)
   */
  public function setSheetName(string $name) {
    $this->sheetName = $name;
  }

  /**
   * Transforms the data to a JSON string.
   *
   * @param array $data
   *   The data of the sheet.
   */
  public function toJson(array $data) {
    $rows = [];
    $maxColumns = max(array_map(function ($row) {
      return count($row);

    }, $data));
    foreach ($data as $key => $row) {
      for ($i = 0; $i < $maxColumns; $i++) {
        $rows[$key][$i] = (!empty($row[$i])) ? $row[$i] : '';
      }
    }
    return json_encode($rows, JSON_PRETTY_PRINT);
  }

  /**
   * Transforms the sheet data into a render array.
   *
   * @param array $data
   *   The data of the sheet.
   * @param bool $setheader
   *   Sets the 1st line as the table header.
   */
  public function render(array $data, bool $setheader = TRUE) {
    $header = [];
    $rows = [];
    $maxColumns = max(array_map(function ($row) {
      return count($row);

    }, $data));
    if ($setheader) {
      for ($i = 0; $i < $maxColumns; $i++) {
        $header[$i] = (!empty($data[0][$i])) ? $data[0][$i] : '';
      }
      unset($data[0]);
    }
    foreach ($data as $key => $row) {
      for ($i = 0; $i < $maxColumns; $i++) {
        $rows[$key][$i] = (!empty($row[$i])) ? $row[$i] : '';
      }
    }
    return [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
    ];
  }

  /**
   * Checks if myArray contains a word as key.
   *
   * @param array $myArray
   *   The array to check.
   * @param mixed $word
   *   The word to find.
   *
   * @return bool
   *   True if the word is found false otherwise
   */
  public function myArrayContainsWord(array $myArray, $word) {
    foreach ($myArray as $element) {
      if ($element->title == $word) {
        return TRUE;
      }
    }
    return FALSE;
  }

}
